# Описание
DITranquallity - небольшая библиотека для внедрения зависимостей в приложениях, написанных на чистом Swift. Несмотря на её размер, она решает достаточно большой спектр задач, в том числе поддерживает Storyboard. Главное её преимущество над остальными - поддержка модульности, наличие ошибок с описание и большое количество возможностей.

# Особенности

* Поддержка чистых Swift типов
* Внедрение зависимостей через: [метод инициализации](registration.md#Разрешение-зависимостей-при-инициализации), [свойства, метод](registration.md#Разрешение-зависимостей-после-инициализации)
* [Объявление имени типа](resolve.md#По-типу-и-имени)
* [Поддержка указания нескольких дочерних типов](registration.md#Указание-дочерних-типов)
* [5 времен жизни: single, lazySingle, weakSingle, perScope, perDependency](lifetime.md#Время-жизни)
* [iOS/macOS Storyboard](storyboard.md#storyboard)
* [Внедрение с рантайм аргументами](injection.md#Внедрение-с-рантайм-аргументами)
* [Поддержка циклических зависимостей](injection.md#Циклические-ссылки)
* Несколько уровней абстракций: тип, [компонент](component.md#Компоненты), [модуль](module.md#Модули)
* [Поддержка позднего связывания и область видимости компонетов](lateBinding.md#Позднее-связывание)
* [Краткий синтаксис получения объекта](resolve.md#Сокращенный-синтаксис)
* Безопасный резолвинг из нескольких потоков
* [Поиск компонент, модулей](scan.md#Поиск)
* [9 видов ошибок. Каждая ошибка имеет подробное описание](errors.md#Исключения). [Логирование](log.md#Логирование)
* [Автоматическое внедрение зависимостей через свойства для Obj-C типов](injection.md#Автоматическое-внедрение-зависимостей-через-свойства)

# Главы

## [Быстрый старт](quick_start.md#Быстрый-старт)
* [Знакомство с идеей "инверсии зависимостей" и "внедрения зависимостей"](quick_start.md#Знакомство-с-идеей-инверсии-зависимостей-и-внедрения-зависимостей)
* [Добавление DITranquallity в ваш проект](quick_start.md#Добавление-ditranquillity-в-ваш-проект)
* [Регистрация](quick_start.md#Регистрация)
* [Разрешение зависимостей](quick_start.md#Разрешение-зависимостей)
* [Что дальше?](quick_start.md#Что-дальше)

## [Регистрация](registration.md#Регистрация)
* [Регистрация типа](registration.md#Регистрация-типа)
* [Указание метода инициализации](registration.md#Указание-метода-инициализации)
* [Указание дочерних типов](registration.md#Указание-дочерних-типов)
* [Разрешение зависимостей при инициализации](registration.md#Разрешение-зависимостей-при-инициализации)
* [Разрешение зависимостей после инициализации](registration.md#Разрешение-зависимостей-после-инициализации)

## [Внедрение](injection.md#Внедрение)
* [Внедрение зависимостей](injection.md#Внедрение-зависимостей)
* [Автоматическое внедрение зависимостей через свойства](injection.md#Автоматическое-внедрение-зависимостей-через-свойства)
* [Циклические ссылки](injection.md#Циклические-ссылки)
* [Внедрение с рантайм аргументами](injection.md#Внедрение-с-рантайм-аргументами)

## [Создание контейнера](build.md#Создание-контейнера)
* [Синтаксис](build.md#Синтаксис)
* [Процесс создания](build.md#Процесс-создания)
* [Проверки и исключения](build.md#Проверки-и-исключения)

## [Разрешение зависимостей](resolve.md#Разрешение-зависимостей)
* [По типу](resolve.md#По-типу)
* [По типу и имени](resolve.md#По-типу-и-имени)
* [По умолчанию](resolve.md#По-умолчанию)
* [Множественная](resolve.md#Множественная)
* [Передача параметров](resolve.md#Передача-параметров)
* [Автоматический вывод типов](resolve.md#Автоматический-вывод-типов)
* [Для существующего объекта](resolve.md#Для-существующего-объекта)
* [Сокращенный синтаксис](resolve.md#Сокращенный-синтаксис)
* [Проверки и исключения](resolve.md#Проверки-и-исключения)

## [Указанием имени/множественная регистрация](multi_name_registration.md#Указанием-именимножественная-регистрация)
* [Множественная регистрация](multi_name_registration.md#Множественная-регистрация)
* [Указание зависимости по умолчанию](multi_name_registration.md#Указание-зависимости-по-умолчанию)
* [Указание имени](multi_name_registration.md#Указание-имени)

## [Время жизни](lifetime.md#Время-жизни)
* [Одиночка (single)](lifetime.md#Одиночка-single)
* [Отложенная одиночка (lazySingle)](lifetime.md#Отложенная-одиночка-lazysingle)
* [Слабая одиночка (weakSingle)](lifetime.md#Слабая-одиночка-weaksingle)
* [Область видимости (perScope)](lifetime.md#Область-видимости-perscope)
* [Всегда новый (perDependency)](lifetime.md#Всегда-новый-perdependency)

## [Компоненты](component.md#Компоненты)
* [Объявление](component.md#Объявление)
* [Регистрация](component.md#Регистрация)
* [Область видимости](component.md#Область-видимости)

## [Модули](module.md#Модули)
* [Объявление](module.md#Объявление)
* [Регистрация](module.md#Регистрация)

## [Позднее связывание](lateBinding.md#Позднее-связывание)
* [Объявление](lateBinding.md#Объявление)
* [Объявление реализации](lateBinding.md#Объявление-реализации)
* [На что стоит обратить внимание](lateBinding.md#На-что-стоит-обратить-внимание)
* [Ошибки](lateBinding.md#Ошибки)

## [Storyboard](storyboard.md#storyboard)
* [Регистрация ViewController](storyboard.md#Регистрация-viewcontroller)
* [Сокращенный синтаксис](storyboard.md#Сокращенный-синтаксис)
* [Создание Storyboard](storyboard.md#Создание-storyboard)
* [Простое создание Storyboard](storyboard.md#Простое-создание-storyboard)


## [Поиск](scan.md#Поиск)
* [Предыстория](scan.md#Предыстория)
* [Поиск модулей](scan.md#Поиск-модулей)
* [Поиск компонент](scan.md#Поиск-компонент)
* [Указание Bundle](scan.md#Указание-bundle)
* [Возможности](scan.md#Возможности)

## [Логирование](log.md#Логирование)
* [Использование](log.md#Использование)
* [События](log.md#События)

## [Исключения](errors.md#Исключения)

## [Примеры](sample.md#Примеры)
* [Chaos](sample.md#chaos)
* [Delegate and Observer](sample.md#delegate-and-observer)
* [Habr](sample.md#habr)
* [OSX](sample.md#osx)
* ~~Big Project~~

# [Словарик](glossary.md#Словарик)